import numpy as np
from rich import print


def detect_top(array, cy, space, threshold):
    height, _, _ = array.shape
    for end_h in range(cy, 0, -space):
        start_h = max(end_h - threshold, 0)
        if np.count_nonzero(array[start_h:end_h, :, :]) == 0:
            return end_h
    return 0


def detect_bottom(array, cy, space, threshold):
    height, _, _ = array.shape
    for start_h in range(cy, height, space):
        end_h = min(start_h + threshold, height)
        if np.count_nonzero(array[start_h:end_h, :, :]) == 0:
            return start_h
    return height


def detect_left(array, cx, top, bottom, space, threshold):
    _, width, _ = array.shape
    for end_w in range(cx, 0, -space):
        start_w = max(end_w - threshold, 0)
        nnz = np.count_nonzero(array[top:bottom, start_w:end_w, :])
        if nnz == 0:
            return end_w
    return 0


def detect_right(array, cx, top, bottom, space, threshold):
    _, width, _ = array.shape
    for start_w in range(cx, width, space):
        end_w = min(start_w + threshold, width)
        nnz = np.count_nonzero(array[top:bottom, start_w:end_w, :])
        if nnz == 0:
            return start_w
    return width


def detect_center(array):
    height, width, _ = array.shape
    center_x = -1
    center_y = -1
    non_zero = -1
    for i in range(height):
        nnz = np.count_nonzero(array[i : i + 1, :, :])
        if nnz > non_zero:
            center_y = i
            non_zero = nnz
    non_zero = -1
    for j in range(width):
        nnz = np.count_nonzero(array[:, j : j + 1, :])
        if nnz > non_zero:
            center_x = j
            non_zero = nnz
    return center_x, center_y


def detect_border(array, space, threshold_w, threshold_h):
    height, width, _ = array.shape
    top, right, bottom, left = (0, 0, 0, 0)
    # detect the top border
    for start_h in range(0, height, space):
        end_h = min(start_h + threshold_h, height)
        nnz = np.count_nonzero(array[start_h:end_h, :, :])
        if nnz != 0:
            top = min(start_h, height)
            break
    # detect the bottom border
    for end_h in range(height, top, -space):
        start_h = max(end_h - threshold_h, 0)
        nnz = np.count_nonzero(array[start_h:end_h, :, :])
        if nnz != 0:
            bottom = min(end_h, height)
            break
    # detect the left border
    for start_w in range(0, width, space):
        end_w = min(start_w + threshold_w, width)
        nnz = np.count_nonzero(array[:, start_w:end_w, :])
        if nnz != 0:
            left = min(start_w, width)
            break
    # detect the right border
    for end_w in range(width, left, -space):
        start_w = max(end_w - threshold_w, 0)
        nnz = np.count_nonzero(array[:, start_w:end_w, :])
        if nnz != 0:
            right = min(end_w, width)
            break
    return top, bottom, left, right


def detect_frame(array, space, threshold, crop_from='center'):
    reversed_array = array - 255
    height, width, _ = array.shape
    threshold_h = min(max(int(height * threshold), 1), space)
    threshold_w = min(max(int(width * threshold), 1), space)

    if crop_from == 'center':
        cx, cy = detect_center(reversed_array)
        top = detect_top(reversed_array, cy, space, threshold_h)
        bottom = detect_bottom(reversed_array, cy, space, threshold_h)
        left = detect_left(reversed_array, cx, top, bottom, space, threshold_w)
        right = detect_right(reversed_array, cx, top, bottom, space, threshold_w)
    else: # now only support border
        top, bottom, left, right = detect_border(reversed_array, space, threshold_w, threshold_h)
    return left / width, top / height, right / width, bottom / height


def rect_str(rect):
    return "(%d, %d), (%d, %d)" % (rect[0], rect[1], rect[2], rect[3])



import argparse
from rich.console import Console
from rich.table import Table
from pdf_crop.crop_pymupdf import PymuPDFCrop
from pdf_crop.helper import rect_str


def parse_arguments():
    common_parser = argparse.ArgumentParser(add_help=True)
    common_parser.add_argument(
        "-i", "--input_file", type=str, required=True, help="Input PDF"
    )
    common_parser.add_argument(
        "-p",
        "--page",
        action="append",
        nargs=6,
        metavar=("number", "left", "top", "right", "bottom", "output"),
        required=False,
        help="Which pages to crop",
    )
    common_parser.add_argument("--space", type=int, default=5, help="Detection space")
    common_parser.add_argument(
        "--threshold", type=float, default=0.008, help="White threshold to recognize a border"
    )

    return common_parser.parse_known_args()


def validate_flags(flags):
    pass


def main():
    flags, unparsed = parse_arguments()
    validate_flags(flags)
    input_file = flags.input_file
    page_option = flags.page

    table = Table(title="PDF Crop Summary", leading=0)
    table.add_column("Page No.", justify="right", style="cyan", no_wrap=True)
    table.add_column("MediaBox", justify="right", style="magenta")
    table.add_column("CropBox", justify="right", style="magenta")
    table.add_column("Output", justify="right", style="green")

    pdf_crop = PymuPDFCrop(input_file=input_file, space=flags.space, threshold=flags.threshold)
    if page_option is None:
        for i in range(pdf_crop.page_count()):
            output_file, media_box, crop_box = pdf_crop.crop(i)
            table.add_row(str(i), rect_str(media_box), rect_str(crop_box), output_file)
    else:
        for option in page_option:
            page_no = int(option[0])
            left = int(option[1])
            top = int(option[2])
            right = int(option[3])
            bottom = int(option[4])
            output_file = option[5]
            if left == right or top == bottom:
                crop_box = None
            else:
                crop_box = (left, top, right, bottom)
            output_file, media_box, crop_box = pdf_crop.crop(
                page_no, output_file=output_file, crop_box=crop_box
            )
            table.add_row(
                option[0], rect_str(media_box), rect_str(crop_box), output_file
            )
    # Show summary
    console = Console()
    console.print(table)


if __name__ == "__main__":
    main()



import os
import tempfile

import fitz
from rich import print
import imageio

from pdf_crop import helper


class PymuPDFCrop(object):
    def __init__(self, input_file, dpi=128, threshold=0.008, space=2, crop_from='center'):
        self._input_file = input_file
        self._input_pdf = fitz.open(input_file)
        self._page_count = self._input_pdf.page_count
        self._dpi = dpi
        self._space = space
        self._threshold = threshold
        self._crop_from = crop_from

    def crop(self, page_no, output_file=None, crop_box=None):
        if page_no >= self._page_count:
            print("[red]page no exceed the page number[/red]")
            return
        page = self._input_pdf[page_no]
        if crop_box is None:
            crop_box = self.detect_margin(page)
        page.set_cropbox(crop_box)

        return self.save_pdf(page, output_file), page.mediabox, crop_box

    def page_count(self):
        return self._page_count

    def detect_margin(self, page):
        # Save temporary png file
        tmp_png = tempfile.gettempdir() + os.path.sep + "detect_crop_box.png"
        pix = page.get_pixmap(dpi=self._dpi)
        pix.save(tmp_png)
        origin_rect = page.mediabox
        media_width = origin_rect[2] - origin_rect[0]
        media_height = origin_rect[3] - origin_rect[1]

        image = imageio.imread(tmp_png)
        detect_rect = helper.detect_frame(image, self._space, self._threshold, self._crop_from)

        result_rect = [
            int(detect_rect[0] * media_width),
            int(detect_rect[1] * media_height),
            int(detect_rect[2] * media_width),
            int(detect_rect[3] * media_height),
        ]
        return fitz.Rect(result_rect)

    def save_pdf(self, page, output_file):
        page_no = page.number
        output_pdf = fitz.open()
        output_pdf.insert_pdf(self._input_pdf, from_page=page_no, to_page=page_no)
        if output_file is None or len(output_file) == 0:
            input_prefix = os.path.splitext(self._input_file)[0]
            output_file = f"{input_prefix} - page {page_no:2d}.pdf"
        output_pdf.save(output_file)
        return output_file



import os
from os import walk

from rich.console import Console
from rich.table import Table
from pdf_crop.crop_pymupdf import PymuPDFCrop
from pdf_crop.helper import rect_str


def crop(input_file, page_option: list, show_summary=True, crop_from='center'):
    table = Table(title="PDF Crop Summary", leading=0)
    table.add_column("Page No.", justify="right", style="cyan", no_wrap=True)
    table.add_column("MediaBox", justify="right", style="magenta")
    table.add_column("CropBox", justify="right", style="magenta")
    table.add_column("Output", justify="right", style="green")
    table_data = list()

    space, threshold = (5, 0.008)
    for option in page_option:
        page_no, output_file = option[:2]
        if len(option) > 2:
            space = option[2]
        if len(option) > 3:
            threshold = option[3]
        pdf_crop = PymuPDFCrop(input_file=input_file, space=space, threshold=threshold, crop_from=crop_from)
        output_file, media_box, crop_box = pdf_crop.crop(page_no, output_file=output_file, crop_box=None)
        table_data.append((str(option[0]), rect_str(media_box), rect_str(crop_box), output_file))
    for row in table_data:
        table.add_row(*row)
    if show_summary:
        console = Console()
        console.print(table)
    return table_data


def crop_single_page_files(input_folder, output_folder, show_summary=True, space=5, threshold=0.008):
    if not os.path.exists(output_folder):
        os.mkdir(output_folder)

    pdfs = []
    for (dirpath, dirnames, filenames) in walk(input_folder):
        pdfs.extend(filenames)
        break
    summary = list()
    for f in pdfs:
        info = crop(f"{input_folder}/{f}",[(0, f"{output_folder}/{f}", 6), space, threshold], show_summary=False)
        summary.append((f"{input_folder}/{f}", *info[0][1:]))

    table = Table(title="PDF Crop Summary", leading=0)
    table.add_column("Input", justify="right", style="cyan", no_wrap=True)
    table.add_column("MediaBox", justify="right", style="magenta")
    table.add_column("CropBox", justify="right", style="magenta")
    table.add_column("Output", justify="right", style="green")
    for row in summary:
        table.add_row(*row)
    if show_summary:
        console = Console()
        console.print(table)